import argparse

from typing import List
from pathlib import Path

import numpy as np
import cv2 as cv

from tpcwj import __version__
from tpcwj.utils import Bytes, Utils
from tpcwj.packer import Packer
from tpcwj.unpacker import Unpacker


class TPCWJ:
    def __init__(self, opts):
        self.opts = opts

    @classmethod
    def unpack(cls, image_paths: List[Path]) -> None:
        images = []
        for image_path in image_paths:
            image_data = cv.imread(str(image_path))
            image_data = cv.cvtColor(image_data, cv.COLOR_BGR2RGB)
            images.append(Utils.image2bytes(image_data))
        unpacked_file_name, unpacked_file_data = Unpacker.unpack(images)
        unpacked_file_data.tofile(unpacked_file_name)

    @classmethod
    def pack(cls, file_path: Path, output_dir: Path, slice_amount=1) -> None:
        original_file = np.fromfile(file_path, dtype=np.uint8)
        original_file = Bytes.from_ndarray(original_file)
        packed_images = Packer.pack(file_path.name, original_file, slice_amount)
        file_id = Utils.randstr(8)
        idx = 0
        for packed in packed_images:
            idx = idx+1
            image = Utils.bytes2image(packed)
            image = cv.cvtColor(image, cv.COLOR_RGB2BGR)
            file_name = Path(f'TPCWJ-{file_id}-{idx}.png')
            file_path = output_dir/file_name
            cv.imwrite(str(file_path), image)

    def run(self) -> int:
        if self.opts.action == 'unpack':
            self.unpack([Path(x)for x in self.opts.image_paths])
        elif self.opts.action == 'pack':
            self.pack(Path(self.opts.file_path), Path(self.opts.output_dir), int(self.opts.slice))
        return 0


def run() -> int:
    parser = argparse.ArgumentParser(description=f'TPCWJ v{__version__} - a command line tool that pack file into images.')
    parser.add_argument('-v', '--version', action='version', help='show current version installed', version=f'TPCWJ v{__version__}')
    subcommands = parser.add_subparsers(title='subcommands', dest='action')
    pack_parser = subcommands.add_parser(name='pack', help='pack file into images')
    pack_parser.add_argument('-s', '--slice', metavar='slice_num', help='specify number of slices', default=1)
    pack_parser.add_argument('-o', '--output-dir', metavar='output_dir', help='specify packed images output dir', default='.')
    pack_parser.add_argument('file_path', help='specify path of target file')

    unpack_parser = subcommands.add_parser(name='unpack', help='unpack payloads in images into file')
    unpack_parser.add_argument('image_paths', nargs='+', help='specify path of host images')

    opts = parser.parse_args()
    tpcwj = TPCWJ(opts)
    return tpcwj.run()
